import { writeFile } from 'fs/promises';
import semver from 'semver';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 *
 */
export async function syncDependencies(param) {
    const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    const { default: packageJson } = await import(packageJsonPath, { assert: { type: 'json' } });
    if (!packageJson.dependencies) {
        packageJson.dependencies = {};
    }
    const missingDependenciesFromDeepJson = deepJson.dependencies.filter((dependency) => !!packageJson.dependencies[dependency.name]);
    missingDependenciesFromDeepJson.forEach((dependency) => {
        packageJson.dependencies = { ...packageJson.dependencies, [dependency.name]: `~${dependency.version}` };
    });
    if (missingDependenciesFromDeepJson.length > 0) {
        await writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
        console.log(`${missingDependenciesFromDeepJson.map((dependency) => dependency.name).join(', ')} are added to package.json because they exist in deep.json`);
    }
    deepJson.dependencies.forEach((dependency) => {
        const isDeepJsonVersionGreater = semver.gt(dependency.version, packageJson.dependencies[dependency.name]);
        if (isDeepJsonVersionGreater) {
            packageJson.dependencies[dependency.name] = `~${dependency.version}`;
        }
        else {
            deepJson.dependencies = { ...deepJson.dependencies, [dependency.name]: `${dependency.version}` };
        }
    });
    Object.entries(packageJson.dependencies).forEach(([dependencyName, dependencyVersion]) => {
        const deepJsonDependency = deepJson.dependencies.find(dependency => dependency.name === dependency.name);
        if (!deepJsonDependency)
            return;
        const isPackageJsonVersionGreater = semver.gt(dependencyVersion, deepJson.dependencies.find(dependency => dependency.name === dependency.name).version);
        if (isPackageJsonVersionGreater) {
            deepJson.dependencies = { ...deepJson.dependencies, [dependencyName]: `${dependencyVersion}` };
        }
        else {
            packageJson.dependencies[dependencyName] = `~${deepJsonDependency.version}`;
        }
    });
}
